package java.time.temporal;

import java.time.DateTimeException;
import java.util.Objects;

public interface TemporalAccessor
{
    public abstract boolean isSupported(TemporalField p0);
    
    default public ValueRange range(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            if (this.isSupported(temporalField)) {
                return temporalField.range();
            }
            throw new UnsupportedTemporalTypeException("Unsupported field: " + temporalField);
        }
        else {
            Objects.requireNonNull(temporalField, "field");
            return temporalField.rangeRefinedBy(this);
        }
    }
    
    default public int get(TemporalField temporalField) {
        ValueRange range = this.range(temporalField);
        if (!range.isIntValue()) {
            throw new UnsupportedTemporalTypeException("Invalid field " + temporalField + " for get() method, use getLong() instead");
        }
        long long1 = this.getLong(temporalField);
        if (!range.isValidValue(long1)) {
            throw new DateTimeException("Invalid value for " + temporalField + " (valid values " + range + "): " + long1);
        }
        return (int)long1;
    }
    
    public abstract long getLong(TemporalField p0);
    
    default public <R> R query(TemporalQuery<R> temporalQuery) {
        if (temporalQuery == TemporalQueries.zoneId() || temporalQuery == TemporalQueries.chronology() || temporalQuery == TemporalQueries.precision()) {
            return null;
        }
        return temporalQuery.queryFrom(this);
    }
}
